==> abc-cli-0.2.37.tgz: package/utils.js <==
/**
 * @fileOverview 工具方法
 * @author langford@126.com
 */

const path = require('path')
const _ = require('lodash')

const browsers = ['>1%', 'ie > 8']

const paths = {
  builderNodeModulePath: path.resolve(__dirname, './node_modules'),
  appNodeModulePath: path.resolve(process.cwd(), './node_modules'),
  appRootPath: path.resolve(process.cwd())
}

/**
 * 如果启用了 buildByVersion 选项，修改 output 输出配置， 注意：此函数有副作用，会修复 abcConfig 配置项
 * 会优先读取 package.json 文件中的 version, 如果没有配置，会读取 abc.config.js 中 buildByVersion.version 的版本号
 * @param {Object} abcConfig 合并了默认配置以及abc.config.js后的配置文件
 * @param {Object} package 项目中的 package.json 文件
 */
const _setVersion = (abcConfig) => {
  const package = require(process.cwd() + '/package.json')

  if (_.get(abcConfig, 'buildByVersion.enable', false)) {
    let version = _.get(abcConfig, 'buildByVersion.version', '0.1.0')

    if (package && package.version) {
      version = package.version
    }

    abcConfig.output.path = path.join(abcConfig.output.path, `${version}`)
    abcConfig.outputDir = path.join(abcConfig.outputDir, `${version}`)
  }
}

const abcConfig = ((config = {}) => {
  const defaultConfig = {
    port: 8080,
    host: 'localhost',
    entry: {},
    output: {
      path: paths.appRootPath + '/dist',
      filename: '[name].[hash:5].js',
      chunkFilename: '[name].[chunkhash:5].js',
      publicPath: './'
    },
    rules: [],
    plugins: [],
    alias: {},
    externals: {},
    babelOptions: {},
    browsersList: ['>1%', 'ie > 8'],
    outputDir: 'dist',
    htmlTemplateUrl: path.resolve(__dirname, '.') + '/index.tmpl.html',
    htmlWebpackPluginOptions: {},
    devtool: 'source-map',
    isExtractCss: true,
    proxy: {},
    history: {
      whiteList: []
    },
    staticPath: '',
    minCssExtract: {
      filename: '[name].[hash:5].css',
      chunkFilename: '[id].[hash:5].css'
    },
    cssModule: {
      enable: false,
      localIdentName: `[local]-[hash:base64:5]`
    },
    disableErrorOverlay: false,
    env: {
      development: {
        rules: [],
        plugins: [],
        entry: {},
        output: {},
        babelOptions: {},
        htmlTemplateUrl: '',
        htmlWebpackPluginOptions: {},
        devtool: 'source-map'
      },
      daily: {
        rules: [],
        plugins: [],
        entry: {},
        output: {},
        babelOptions: {},
        htmlTemplateUrl: '',
        htmlWebpackPluginOptions: {},
        devtool: 'eval-source-map'
      },
      production: {
        rules: [],
        plugins: [],
        entry: {},
        output: {},
        babelOptions: {},
        htmlTemplateUrl: '',
        htmlWebpackPluginOptions: {},
        devtool: 'eval-source-map'
      }
    },
    dll: {
      enable: false,
      vendors: ['react']
    },
    cleanBuild: true,
    analyzerReport: false,
    buildNotifier: {
      title: 'abc build',
      show: true,
      timeout: 5
    },
    buildByVersion: {
      enable: false,
      version: '0.1.0'
    }
  }

  try {
    const abcConfig = require(process.cwd() + '/abc.config.js')
    const mergeConfig = _.merge({}, defaultConfig, abcConfig, config)

    _setVersion(mergeConfig)

    return mergeConfig
  } catch (ex) {
    console.log(`abc.config.js error => ${ex}`)
    return _.merge({}, defaultConfig, config)
  }
})()

const getBabelOptions = (config = {}, envConfig = {}) => {
  const { dll } = abcConfig

  let plugins = []

  // 未启用 dll 打包 或者启用了 dll 打包，但是 vendors 中不包含 antd 配置
  if (!dll.enable || (dll.enable && dll.vendors.indexOf('antd') <= 0)) {
    plugins = [
      [
        require.resolve('babel-plugin-import'), // antd 模块按需加载
        [{ libraryName: 'antd', libraryDirectory: 'lib' }]
      ],
      ...plugins
    ]
  }

  plugins = [
    ...plugins,
    require.resolve('babel-plugin-transform-decorators-legacy'),
    require.resolve('babel-plugin-transform-runtime'),
    require.resolve('babel-plugin-transform-class-properties'),
    require.resolve('babel-plugin-add-module-exports'),
    require.resolve('babel-plugin-transform-object-rest-spread'),
    require.resolve('babel-plugin-lodash'),
    require.resolve('babel-plugin-date-fns')
  ]

  const babelOptions = {
    babelrc:
      _.get(envConfig, 'babelOptions.babelrc', false) ||
      _.get(abcConfig, 'babelOptions.babelrc', false) ||
      false,
    cacheDirectory:
      _.get(envConfig, 'babelOptions.cacheDirectory', '') ||
      _.get(abcConfig, 'babelOptions.cacheDirectory', '') ||
      true,
    presets: merge(
      [
        envConfig.babelOptions.presets || [],
        abcConfig.babelOptions.presets || [],
        [
          [
            require.resolve('babel-preset-env'),
            {
              targets: {
                browsers: browsers
              },
              debug: false,
              useBuiltIns: true,
              module: false
            }
          ],
          require.resolve('babel-preset-stage-0'),
          require.resolve('babel-preset-react')
        ]
      ],
      'preset'
    ),
    plugins: merge(
      [
        envConfig.babelOptions.plugins || [],
        abcConfig.babelOptions.plugins || [],
        plugins
      ],
      'preset'
    )
  }

  // 开发模式加载 hmr 插件
  if (config.dev && babelOptions.presets) {
    babelOptions.presets.push(require.resolve('babel-preset-react-hmre'))
  }

  return babelOptions
}

// IE9+,firefox,chrome等
const getPostCssOptions = (config = {}) => {
  return _.merge(
    {},
    {
      plugins: [
        require('autoprefixer')({
          browsers: abcConfig.browsersList || browsers
        }),
        require('cssnano')()
      ]
    },
    config
  )
}

const getDevEntry = (entry) => {
  const arr = []

  if (_.isString(entry) || _.isArray(entry)) {
    return { app: _.clone(arr).concat(entry) }
  }

  if (_.isObject(entry)) {
    _.mapKeys(entry, (value, key) => {
      entry[key] = _.clone(arr).concat(value)
    })
  }

  return entry
}

/**
 * 处理代理配置
 * @param {Object} proxyConfig
 *
 * proxyConfig = {
 *  '/ecs': 'https://api.aliyun.com/',
 *  '/example/:id': params => {
 *     return {
 *       target: 'http://localhost:8000',
 *       logs: true,
 *       headers: {
 *         'X_HOST_S': 'google.com'
 *       }
 *     }
 *   }
 * }
 */
const getProxyConfig = () => {
  const result = []
  const proxyConfig = abcConfig.proxy

  if (_.isArray(proxyConfig)) return proxyConfig

  _.keys(proxyConfig).forEach((context) => {
    let options = proxyConfig[context]
    if (typeof options === 'string') {
      options = {
        target: options,
        changeOrigin: true,
        logs: true
      }
    }

    result.push({
      context,
      ...options
    })
  })

  return result
}

const mergeRules = (srcRules, destRules) => {
  const differenceRules = _.differenceBy(srcRules, destRules, function(it) {
    return it.test.toString()
  })

  return _.concat(differenceRules, destRules)
}

const mergePlugins = (srcPlugins, destPlugins) => {
  const differencePlugins = _.differenceBy(srcPlugins, destPlugins, function(
    it
  ) {
    return _.get(it, 'constructor.name', '')
  })

  return _.concat(differencePlugins, destPlugins)
}

const mergePresets = (srcPresets, destPresets) => {
  let item

  const differencePresets = _.differenceBy(srcPresets, destPresets, function(
    it
  ) {
    item = it
    if (_.isArray(it)) {
      item = it[0]
    }

    const tmp = item.toString().split('@')

    return tmp[tmp.length - 1]
  })

  return _.concat(differencePresets, destPresets)
}

const merge = (arr, type = 'rule') => {
  const typeMap = {
    rule: mergeRules,
    plugin: mergePlugins,
    preset: mergePresets
  }

  return arr.reduce((result, it) => {
    return (result = typeMap[type](result, it))
  }, [])
}

module.exports = {
  getBabelOptions,
  getPostCssOptions,
  paths,
  abcConfig,
  getDevEntry,
  getProxyConfig,
  browsers,
  mergeRules,
  mergePlugins,
  mergePresets,
  merge
}

==> abc-cli-0.2.46.tgz: package/utils.js <==
/**
 * @fileOverview 工具方法
 * @author langford@126.com
 */

const path = require("path")
const argv = require("yargs").argv
const _ = require("lodash")

const browsers = [">1%", "ie > 8"]

const paths = {
  builderNodeModulePath: path.resolve(__dirname, "./node_modules"),
  appNodeModulePath: path.resolve(process.cwd(), "./node_modules"),
  appRootPath: path.resolve(process.cwd())
}

/**
 * 如果启用了 buildByVersion 选项，修改 output 输出配置， 注意：此函数有副作用，会修复 abcConfig 配置项
 * 会优先读取 package.json 文件中的 version, 如果没有配置，会读取 abc.config.js 中 buildByVersion.version 的版本号
 * @param {Object} abcConfig 合并了默认配置以及abc.config.js后的配置文件
 * @param {Object} package 项目中的 package.json 文件
 */
const _setVersion = abcConfig => {
  const package = require(process.cwd() + "/package.json")

  if (_.get(abcConfig, "buildByVersion.enable", false)) {
    let version = _.get(abcConfig, "buildByVersion.version", "0.1.0")

    if (package && package.version) {
      version = package.version
    }

    abcConfig.output.path = path.join(abcConfig.output.path, `${version}`)
    abcConfig.outputDir = path.join(abcConfig.outputDir, `${version}`)
  }
}

const abcConfig = ((config = {}) => {
  const defaultConfig = {
    port: 8080,
    host: "localhost",
    entry: {},
    output: {
      path: paths.appRootPath + "/dist",
      filename: "[name].[hash:5].js",
      chunkFilename: "[name].[chunkhash:5].js",
      publicPath: "./"
    },
    rules: [],
    plugins: [],
    alias: {},
    externals: {},
    babelOptions: {},
    browsersList: [">1%", "ie > 8"],
    outputDir: "dist",
    htmlTemplateUrl: path.resolve(__dirname, ".") + "/index.tmpl.html",
    htmlWebpackPluginOptions: {},
    enableHtmlWebpackPlugin: true,
    devtool: "source-map",
    isExtractCss: true,
    proxy: {},
    history: {
      whiteList: []
    },
    staticPath: "",
    minCssExtract: {
      filename: "[name].[hash:5].css",
      chunkFilename: "[id].[hash:5].css"
    },
    cssModule: {
      enable: false,
      localIdentName: `[local]-[hash:base64:5]`
    },
    disableErrorOverlay: false,
    env: {
      development: {
        rules: [],
        plugins: [],
        entry: {},
        output: {},
        babelOptions: {},
        htmlTemplateUrl: "",
        htmlWebpackPluginOptions: {},
        devtool: "source-map"
      },
      daily: {
        rules: [],
        plugins: [],
        entry: {},
        output: {},
        babelOptions: {},
        htmlTemplateUrl: "",
        htmlWebpackPluginOptions: {},
        devtool: "eval-source-map"
      },
      production: {
        rules: [],
        plugins: [],
        entry: {},
        output: {},
        babelOptions: {},
        htmlTemplateUrl: "",
        htmlWebpackPluginOptions: {},
        devtool: "eval-source-map"
      }
    },
    dll: {
      enable: false,
      vendors: ["react"]
    },
    cleanBuild: true,
    analyzerReport: false,
    buildNotifier: {
      title: "abc build",
      show: true,
      timeout: 5
    },
    buildByVersion: {
      enable: false,
      version: "0.1.0"
    }
  }

  try {
    const abcConfig = require(path.resolve(
      process.cwd(),
      argv.config || "abc.config.js"
    ))
    const mergeConfig = _.merge({}, defaultConfig, abcConfig, config)

    _setVersion(mergeConfig)

    return mergeConfig
  } catch (ex) {
    console.log(`abc.config.js error => ${ex}`)
    return _.merge({}, defaultConfig, config)
  }
})()

const getBabelOptions = (config = {}, envConfig = {}) => {
  const { dll } = abcConfig

  let plugins = []

  // 未启用 dll 打包 或者启用了 dll 打包，但是 vendors 中不包含 antd 配置
  if (!dll.enable || (dll.enable && dll.vendors.indexOf("antd") <= 0)) {
    plugins = [
      [
        require.resolve("babel-plugin-import"), // antd 模块按需加载
        [{ libraryName: "antd", libraryDirectory: "lib" }]
      ],
      ...plugins
    ]
  }

  plugins = [
    ...plugins,
    require.resolve("babel-plugin-transform-decorators-legacy"),
    require.resolve("babel-plugin-transform-runtime"),
    require.resolve("babel-plugin-transform-class-properties"),
    require.resolve("babel-plugin-add-module-exports"),
    require.resolve("babel-plugin-transform-object-rest-spread"),
    require.resolve("babel-plugin-lodash"),
    require.resolve("babel-plugin-date-fns")
  ]

  const babelOptions = {
    babelrc:
      _.get(envConfig, "babelOptions.babelrc", false) ||
      _.get(abcConfig, "babelOptions.babelrc", false) ||
      false,
    cacheDirectory:
      _.get(envConfig, "babelOptions.cacheDirectory", "") ||
      _.get(abcConfig, "babelOptions.cacheDirectory", "") ||
      true,
    presets: merge(
      [
        envConfig.babelOptions.presets || [],
        abcConfig.babelOptions.presets || [],
        [
          [
            require.resolve("babel-preset-env"),
            {
              targets: {
                browsers: browsers
              },
              debug: false,
              useBuiltIns: true,
              module: false
            }
          ],
          require.resolve("babel-preset-stage-0"),
          require.resolve("babel-preset-react")
        ]
      ],
      "preset"
    ),
    plugins: merge(
      [
        envConfig.babelOptions.plugins || [],
        abcConfig.babelOptions.plugins || [],
        plugins
      ],
      "preset"
    )
  }

  // 开发模式加载 hmr 插件
  if (config.dev && babelOptions.presets) {
    babelOptions.presets.push(require.resolve("babel-preset-react-hmre"))
  }

  return babelOptions
}

// IE9+,firefox,chrome等
const getPostCssOptions = (config = {}) => {
  return _.merge(
    {},
    {
      plugins: [
        require("autoprefixer")({
          browsers: abcConfig.browsersList || browsers
        }),
        require("cssnano")()
      ]
    },
    config
  )
}

const getDevEntry = entry => {
  const arr = []

  if (_.isString(entry) || _.isArray(entry)) {
    return { app: _.clone(arr).concat(entry) }
  }

  if (_.isObject(entry)) {
    _.mapKeys(entry, (value, key) => {
      entry[key] = _.clone(arr).concat(value)
    })
  }

  return entry
}

/**
 * 处理代理配置
 * @param {Object} proxyConfig
 *
 * proxyConfig = {
 *  '/ecs': 'https://api.aliyun.com/',
 *  '/example/:id': params => {
 *     return {
 *       target: 'http://localhost:8000',
 *       logs: true,
 *       headers: {
 *         'X_HOST_S': 'google.com'
 *       }
 *     }
 *   }
 * }
 */
const getProxyConfig = () => {
  const result = []
  const proxyConfig = abcConfig.proxy

  if (_.isArray(proxyConfig)) return proxyConfig

  _.keys(proxyConfig).forEach(context => {
    let options = proxyConfig[context]
    if (typeof options === "string") {
      options = {
        target: options,
        changeOrigin: true,
        logs: true
      }
    }

    result.push({
      context,
      ...options
    })
  })

  return result
}

const mergeRules = (srcRules, destRules) => {
  const differenceRules = _.differenceBy(srcRules, destRules, function(it) {
    return it.test.toString()
  })

  return _.concat(differenceRules, destRules)
}

const mergePlugins = (srcPlugins, destPlugins) => {
  const differencePlugins = _.differenceBy(srcPlugins, destPlugins, function(
    it
  ) {
    return _.get(it, "constructor.name", "")
  })

  return _.concat(differencePlugins, destPlugins)
}

const mergePresets = (srcPresets, destPresets) => {
  let item

  const differencePresets = _.differenceBy(srcPresets, destPresets, function(
    it
  ) {
    item = it
    if (_.isArray(it)) {
      item = it[0]
    }

    const tmp = item.toString().split("@")

    return tmp[tmp.length - 1]
  })

  return _.concat(differencePresets, destPresets)
}

const merge = (arr, type = "rule") => {
  const typeMap = {
    rule: mergeRules,
    plugin: mergePlugins,
    preset: mergePresets
  }

  return arr.reduce((result, it) => {
    return (result = typeMap[type](result, it))
  }, [])
}

module.exports = {
  getBabelOptions,
  getPostCssOptions,
  paths,
  abcConfig,
  getDevEntry,
  getProxyConfig,
  browsers,
  mergeRules,
  mergePlugins,
  mergePresets,
  merge
}
